//
// Reticle.cpp
//

// Engine includes.
#include "Config.h"
#include "DisplayManager.h"
#include "GameManager.h"
#include "WorldManager.h"
#include "utility.h"

// Game includes.
#include "demo.h"
#include "Reticle.h"

Reticle::Reticle() {

  // Base reticle settings.
  setType("Reticle");
  setSolidness(df::SPECTRAL);
  setAltitude(df::MAX_ALTITUDE); // Make Reticle in the foreground.

  // Reticle responds to mouse so register.
  registerInterest(df::MSE_EVENT);

  // User can do some stuff with keyboard, so register.
  registerInterest(df::KEYBOARD_EVENT);

  // Start Reticle in center of window.
  df::Vector p(WM.getBoundary().getHorizontal()/2,
	       WM.getBoundary().getVertical()/2);
  setPosition(p);

  // Settings for particles Reticle creates.
  effect = df::SMOKE;
  precipitation = df::UNDEFINED_PRECIPITATION_TYPE;
  color = df::UNDEFINED_COLOR;
  effect_scale = 1.0;
  precipitation_scale = 1.0;
}

// Handle event.
// Return 0 if ignored, else 1.
int Reticle::eventHandler(const df::Event *p_e) {

  if (p_e->getType() == df::MSE_EVENT) {
    const df::EventMouse *p_mouse_event = dynamic_cast <const df::EventMouse *> (p_e);
    mouse(p_mouse_event);
  }

  if (p_e->getType() == df::KEYBOARD_EVENT) {
    const df::EventKeyboard *p_keyboard_event = dynamic_cast <const df::EventKeyboard *> (p_e);
    key(p_keyboard_event);
    return 1;
  }

  // If get here, have ignored this event.
  return 0;
}

// Take appropriate action according to key pressed.
int Reticle::key(const df::EventKeyboard *p_keyboard_event) {

  switch(p_keyboard_event->getKey()) {
  case df::Keyboard::ESCAPE:	// quit
    if (p_keyboard_event->getKeyboardAction() == df::KEY_PRESSED)
      GM.setGameOver();
    break;
  };

  return 1;
}

// Do mouse action.  Return 1 if something done, else 0.
int Reticle::mouse(const df::EventMouse *p_mouse_event) {

  // If mouse not in window, return.
  sf::RenderWindow *p_win = df::DisplayManager::getInstance().getWindow();
  sf::Vector2i lp = sf::Mouse::getPosition(*p_win);
  if (lp.x > df::Config::getInstance().getWindowHorizontalPixels() ||       
    lp.x < 0 ||
      lp.y > df::Config::getInstance().getWindowVerticalPixels() ||
      lp.y < 0) {
    return 0;
  }

  // If moved, change location to new mouse position.
  if (p_mouse_event->getMouseAction() == df::MOVED) {
    setPosition(p_mouse_event->getMousePosition());
    return 1;
  }

  // If mouse pressed in canvas area, create particles.
  if (p_mouse_event->getMouseAction() == df::PRESSED) {

    // If not in canvas, return.
    df::Vector corner(0, 3);
    df::Box b(corner, 
	      WM.getBoundary().getHorizontal(), 
	      WM.getBoundary().getVertical()-7);
    if (!df::boxContainsPosition(b, p_mouse_event -> getMousePosition()))
	return 0;

    // Create specified particles.
    if (effect == df::UNDEFINED_PARTICLE_TYPE)
      addCustomEffect();
    else 
      if (color == df::UNDEFINED_COLOR)
	df::addParticles(effect, getPosition(), effect_scale);
      else 
	df::addParticles(effect, getPosition(), effect_scale, color);
    return 1;
  }

  // If we get here, mouse in window but nothing done.
  return 0;
}

// Draw reticle on window.
int Reticle::draw() {
  return DM.drawCh(getPosition(), RETICLE_CHAR, df::RED); 
}

// Return effect type.
df::ParticleType Reticle::getEffect() const {
  return effect;
}

// Set effect type.
void Reticle::setEffect(df::ParticleType new_effect) {
  effect = new_effect;
}

// Return precipitation type.
df::PrecipitationType Reticle::getPrecipitation() const {
  return precipitation;
}

// Set precipitation type.
void Reticle::setPrecipitation(df::PrecipitationType new_precipitation) {
  precipitation = new_precipitation;
  updatePrecipitation();
}

// Update precipitation if type/scale/direction changed.
void Reticle::updatePrecipitation() {
  static df::PrecipitationType old_precipitation = precipitation;
  static df::Direction old_direction = direction;
  static float old_precipitation_scale = precipitation_scale;

  // static old_direction = precipitation_direction;
  if (old_precipitation != precipitation || 
      old_precipitation_scale != precipitation_scale ||
      old_direction != direction) {

    // Remove any old precipitation particles.
    df::ObjectList obj_list = WM.objectsOfType("Precipitation");
    WM.markForDelete(obj_list);

    // Add new precipitation.
    df::addParticles(precipitation, direction, precipitation_scale);

    // Update values.
    old_precipitation = precipitation;
    old_precipitation_scale = precipitation_scale;
    old_direction = direction;
  }
}

// Return effect scale.
float Reticle::getEffectScale() const {
  return effect_scale;
}

// Set scale.
void Reticle::setEffectScale(float new_effect_scale) {
  effect_scale = new_effect_scale;
}

// Return precipitation scale.
float Reticle::getPrecipitationScale() const {
  return precipitation_scale;
}

// Set precipitation scale.
void Reticle::setPrecipitationScale(float new_precipitation_scale) {
  precipitation_scale = new_precipitation_scale;
  updatePrecipitation();
}

// Return color.
df::Color Reticle::getColor() const {
  return color;
}

// Set color.
void Reticle::setColor(df::Color new_color) {
  color = new_color;
}

// Return direction.
df::Direction Reticle::getDirection() const {
  return direction;
}

// Set direction.
void Reticle::setDirection(df::Direction new_direction) {
  direction = new_direction;
  updatePrecipitation();
}

// Add custom particles.
void Reticle::addCustomEffect() const {

  // Default values, if no match in df-config.txt.
  int number = 1;
  int number_spread = 0;
  float size = 1.0;
  float size_spread = 0.0;
  float speed = 0.3f;
  float speed_spread = 0;
  float age = 100;
  int age_spread = 0;
  unsigned char opacity = 255;
  int opacity_spread = 0;
  df::Vector direction(0,0);
  float direction_spread = 0.0;
  float position_spread = 0.0;
  int r = 255;
  int g = 255;
  int b = 255;
  int color_spread = 0;
  
  // Match number of particles?
  std::string s;
  s = df::Config::getInstance().match("burst");
  if (s != "")
    number = atoi(s.c_str());
  s = df::Config::getInstance().match("burst-spread");
  if (s != "")
    number_spread = atoi(s.c_str());
  
  // Match speed?
  s = df::Config::getInstance().match("speed");
  if (s != "")
    speed = (float) atof(s.c_str());
  s = df::Config::getInstance().match("speed-spread");
  if (s != "")
    speed_spread = (float) atof(s.c_str());
  
  // Match size?
  s = df::Config::getInstance().match("size");
  if (s != "")
    size = (float) atof(s.c_str());
  s = df::Config::getInstance().match("size-spread");
  if (s != "")
    size_spread = (float) atof(s.c_str());
  
  // Match age?
  s = df::Config::getInstance().match("age");
  if (s != "")
    age = (float) atoi(s.c_str());
  s = df::Config::getInstance().match("age-spread");
  if (s != "")
    age_spread = atoi(s.c_str());
  
  // Match opacity?
  s = df::Config::getInstance().match("opacity");
  if (s != "")
    opacity = atoi(s.c_str());
  s = df::Config::getInstance().match("opacity-spread");
  if (s != "")
    opacity_spread = atoi(s.c_str());
  
  // Match color?
  s = df::Config::getInstance().match("color-r");
  if (s != "")
    r = atoi(s.c_str());
  s = df::Config::getInstance().match("color-g");
  if (s != "")
    g = atoi(s.c_str());
  s = df::Config::getInstance().match("color-b");
  if (s != "")
    b = atoi(s.c_str());
  s = df::Config::getInstance().match("color-spread");
  if (s != "")
    color_spread = atoi(s.c_str());
  
  // Match direction?
  s = df::Config::getInstance().match("direction-x");
  if (s != "")
    direction.setX((float) atof(s.c_str()));
  s = df::Config::getInstance().match("direction-y");
  if (s != "")
    direction.setY((float) atof(s.c_str()));
  s = df::Config::getInstance().match("direction-spread");
  if (s != "")
    direction_spread = (float) atof(s.c_str());
  
  // Match position spread?
  s = df::Config::getInstance().match("position-spread");
  if (s != "")
    position_spread = (float) atof(s.c_str());
  
  // Add particles based on above settings.
  df::addParticles(number, number_spread, 
		   getPosition(), position_spread,
		   direction, direction_spread, 
		   size, size_spread,
		   speed, speed_spread,
		   (int) age, age_spread, 
		   opacity, opacity_spread,
		   r, g, b, color_spread);

  return;
}
